@@ -203,7 +203,7 @@ static void send_directory(int f,struct file_list *flist,char *dir)
 
   if (cvs_exclude) {
     strcpy(p,".cvsignore");
-    local_exclude_list = make_exclude_list(fname,NULL);
+    local_exclude_list = make_exclude_list(fname,NULL,0);
   }  
 
   for (di=readdir(d); di; di=readdir(d)) {
@@ -237,11 +237,15 @@ struct file_list *send_file_list(int f,int recurse,int argc,char *argv[])
   if (!flist->files) out_of_memory("send_file_list");
 
   for (i=0;i<argc;i++) {
-    char *fname = argv[i];
+    char fname2[MAXPATHLEN];
+    char *fname = fname2;
+
+    strcpy(fname,argv[i]);
 
     l = strlen(fname);
-    if (l != 1 && fname[l-1] == '/')
-      fname[l-1] = 0;
+    if (l != 1 && fname[l-1] == '/') {
+      strcat(fname,".");
+    }
 
     if (lstat(fname,&st) != 0) {
       fprintf(stderr,"%s : %s\n",fname,strerror(errno));
@@ -253,26 +257,15 @@ struct file_list *send_file_list(int f,int recurse,int argc,char *argv[])
       continue;
     }
 
-    if (S_ISDIR(st.st_mode) && argc == 1) {
-      if (chdir(fname) != 0) {
-	fprintf(stderr,"chdir %s : %s\n",fname,strerror(errno));
-	continue;
-      }
-      send_file_name(f,flist,recurse,".");
-      continue;
-    } 
-
     dir = NULL;
     p = strrchr(fname,'/');
-    if (p && !p[1]) {
-      *p = 0;
-      p = strrchr(fname,'/');
-    }
     if (p) {
       *p = 0;
       dir = fname;
       fname = p+1;      
     }
+    if (!*fname)
+      fname = ".";
 
     if (dir && *dir) {
       if (getcwd(dbuf,MAXPATHLEN-1) == NULL) {
