@@ -42,6 +42,8 @@ int am_server = 0;
 static int sender = 0;
 static int recurse = 0;
 
+static void usage(FILE *f);
+
 static void report(int f)
 {
   int in,out,tsize;
@@ -187,6 +189,46 @@ oom:
 }
 
 
+
+
+static char *get_local_name(struct file_list *flist,char *name)
+{
+  struct stat st;
+
+  if (stat(name,&st) == 0) {
+    if (S_ISDIR(st.st_mode)) {
+      if (chdir(name) != 0) {
+	fprintf(stderr,"chdir %s : %s\n",name,strerror(errno));
+	exit(1);
+      }
+      return NULL;
+    }
+    if (flist->count > 1) {
+      fprintf(stderr,"ERROR: destination must be a diirectory when copying more than 1 file\n");
+      exit(1);
+    }
+    return name;
+  }
+
+  if (flist->count == 1)
+    return name;
+
+  if (mkdir(name,0777) != 0) {
+    fprintf(stderr,"mkdir %s : %s\n",name,strerror(errno));
+    exit(1);
+  }
+
+  if (chdir(name) != 0) {
+    fprintf(stderr,"chdir %s : %s\n",name,strerror(errno));
+    exit(1);
+  }
+
+  return NULL;
+}
+
+
+
+
 void do_server_sender(int argc,char *argv[])
 {
   int i;
@@ -226,11 +268,10 @@ void do_server_sender(int argc,char *argv[])
 
 void do_server_recv(int argc,char *argv[])
 {
-  int pid,i,status;
+  int pid,status;
   char *dir = NULL;
   struct file_list *flist;
-  char *fname=NULL;
-  struct stat st;
+  char *local_name=NULL;
   
   if (verbose > 2)
     fprintf(stderr,"server_recv(%d) starting pid=%d\n",argc,(int)getpid());
@@ -241,73 +282,23 @@ void do_server_recv(int argc,char *argv[])
     argv++;
   }
 
-  if (argc > 0) {
-    fname = argv[0];
-    dir = NULL;
-
-    if (stat(fname,&st) != 0) {
-      if (!recurse || mkdir(fname,0777) != 0) {
-	fprintf(stderr,"stat %s : %s\n",fname,strerror(errno));
-	exit(1);
-      }
-      stat(fname,&st);
-    }
-
-    if (S_ISDIR(st.st_mode)) {
-      dir = fname;
-      fname = NULL;
-    }
-  }
-
-  if (dir) {
-    if (chdir(dir) != 0) {
-      fprintf(stderr,"chdir %s : %s\n",dir,strerror(errno));
-      exit(1);
-    } else {
-      if (verbose > 2) {
-	fprintf(stderr,"server changed to directory %s\n",dir);
-      }
-    }
-  }
-
   flist = recv_file_list(STDIN_FILENO);
-  if (!flist) {
+  if (!flist || flist->count == 0) {
     fprintf(stderr,"nothing to do\n");
     exit(1);
   }
 
+
+  local_name = get_local_name(flist,argv[0]);
+
   if ((pid=fork()) == 0) {
-    recv_files(STDIN_FILENO,flist,fname);
+    recv_files(STDIN_FILENO,flist,local_name);
     if (verbose > 1)
       fprintf(stderr,"receiver read %d\n",read_total());
     exit(0);
   }
 
-  if (verbose > 2)
-    fprintf(stderr,"generator starting pid=%d count=%d\n",
-	    (int)getpid(),flist->count);
-
-  for (i = 0; i < flist->count; i++) {
-    if (S_ISDIR(flist->files[i].mode)) {
-      if (dry_run) continue;
-      if (mkdir(flist->files[i].name,flist->files[i].mode) != 0 && 
-	  errno != EEXIST) {	 
-	fprintf(stderr,"mkdir %s: %s\n",
-		flist->files[i].name,strerror(errno));
-	exit(1);
-      }
-      continue;
-    }
-    fname = flist->files[i].name;
-    if (flist->count == 1 &&
-	argc > 0)
-      fname = argv[0];
-    recv_generator(fname,flist,i,STDOUT_FILENO);
-  }
-  write_int(STDOUT_FILENO,-1);
-  write_flush(STDOUT_FILENO);
-  if (verbose > 1)
-    fprintf(stderr,"generator wrote %d\n",write_total());
+  generate_files(STDOUT_FILENO,flist,local_name);
 
   waitpid(pid, &status, 0);
   exit(status);
@@ -347,7 +338,8 @@ static void usage(FILE *f)
   fprintf(f,"the block size defaults to %d\n",BLOCK_SIZE);  
 }
 
-enum {OPT_VERSION,OPT_SUFFIX,OPT_SENDER,OPT_SERVER,OPT_EXCLUDE,OPT_EXCLUDE_FROM};
+enum {OPT_VERSION,OPT_SUFFIX,OPT_SENDER,OPT_SERVER,OPT_EXCLUDE,
+      OPT_EXCLUDE_FROM};
 
 static char *short_options = "oblpguDCtcahvrne:B:";
 
@@ -379,7 +371,7 @@ static struct option long_options[] = {
 
 int main(int argc,char *argv[])
 {
-    int i, pid, status, pid2, status2;
+    int pid, status, pid2, status2;
     int opt;
     int option_index;
     char *shell_cmd = NULL;
@@ -635,30 +627,12 @@ int main(int argc,char *argv[])
     send_exclude_list(f_out);
 
     flist = recv_file_list(f_in);
-    if (flist->count == 0) {
+    if (!flist || flist->count == 0) {
+      fprintf(stderr,"nothing to do\n");
       exit(0);
     }
 
-    {
-      struct stat st;
-      if (stat(argv[0],&st) != 0) {
-	if (mkdir(argv[0],0777) != 0) {
-	  fprintf(stderr,"mkdir %s : %s\n",argv[0],strerror(errno));
-	  exit(1);
-	}
-	stat(argv[0],&st);
-      }
-      
-      if (S_ISDIR(st.st_mode)) {
-	if (chdir(argv[0]) != 0) {
-	  fprintf(stderr,"chdir %s : %s\n",argv[0],strerror(errno));
-	  exit(1);
-	}
-      } else {
-	local_name = argv[0];
-      }
-    }
-
+    local_name = get_local_name(flist,argv[0]);
 
     if ((pid2=fork()) == 0) {
       recv_files(f_in,flist,local_name);
@@ -667,24 +641,8 @@ int main(int argc,char *argv[])
       exit(0);
     }
 
-    for (i = 0; i < flist->count; i++) {
-      if (S_ISDIR(flist->files[i].mode)) {
-	if (dry_run) continue;
-	if (mkdir(flist->files[i].name,flist->files[i].mode) != 0 &&
-	    errno != EEXIST) {
-	  fprintf(stderr,"mkdir %s : %s\n",
-		  flist->files[i].name,strerror(errno));
-	}
-	continue;
-      }
-      recv_generator(local_name?local_name:flist->files[i].name,
-		     flist,i,f_out);
-    }
-    write_int(f_out,-1);
-    write_flush(f_out);
-    if (verbose > 1)
-      fprintf(stderr,"generator wrote %d\n",write_total());
-    
+    generate_files(f_out,flist,local_name);
+
     waitpid(pid2, &status2, 0);
 
     report(f_in);
